/**
  *  \file flak/flak-db.h
  *
  *  (c) 2001-2006 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_FLAK_FLAK_DB_H
#define PCC_V2_FLAK_FLAK_DB_H

#include <vector>
#include "flak/flak.h"
#include "flak/flakdefs.h"
#include "cpluslib/buffer.h"
#include "game/vcrdb.h"

class Stream;

class FlakVcrEntry {
    FlakBattle    battle;
    TFlakBattle   header;

 public:
    FlakVcrEntry(const Buffer<char>& in);
    ~FlakVcrEntry();

    int getNumShips() const
        { return header.num_ships; }
    int getNumFleets() const
        { return header.num_fleets; }
    int getPositionX() const
        { return header.x; }
    int getPositionY() const
        { return header.y; }
    int32 getTotalTime() const
        { return header.total_time; }
    const TFlakBattle& getHeader() const
        { return header; }

    const FlakBattle& getBattle() const
        { return battle; }
    FlakBattle& getBattle()
        { return battle; }
};

class GFlakVcrDatabase : public GVcrDatabase {
    GFlakVcrDatabase(const GFlakVcrDatabase&);
    GFlakVcrDatabase& operator=(const GFlakVcrDatabase&);

    std::vector<FlakVcrEntry*> battles;

    TFlakHeader last_header;

    bool readOneBattle(Stream& s);
    void loadFromFile(Stream& s);
 public:
    GFlakVcrDatabase();
    GFlakVcrDatabase(Stream& s);
    ~GFlakVcrDatabase();

    /** Get number of battles in this database. */
    int getCount() const
        { return battles.size(); }

    /** Access to battle /nr/ (0-based). */
    FlakVcrEntry& getBattle(int nr)
        { return *battles[nr]; }

    void visit(GVcrDatabaseVisitor& vis);
};

#endif
