/**
  *  \file game/sim-if.cc
  */

#include "game/sim-if.h"
#include "cpluslib/format.h"
#include "util/nls.h"

/*! \class GSimulatorNullInterface
    \brief Null Game/Simulator Interface

    This is a simulator/game interface usable to run the simulator
    standalone, without needing any game data. */

string_t
GSimulatorNullInterface::getPlanetName(int id)
{
    return string_t(format(_("Planet #%d"), id));
}

bool
GSimulatorNullInterface::isGameAvailable()
{
    return false;
}

bool
GSimulatorNullInterface::updateFromGame(GSimShip& /*sh*/)
{
    return false;
}
bool
GSimulatorNullInterface::updateToGame(const GSimShip& /*sh*/)
{
    return false;
}
bool
GSimulatorNullInterface::isGamePlanet(const GSimPlanet& /*pl*/)
{
    return false;
}

bool
GSimulatorNullInterface::updateFromGame(GSimPlanet& /*pl*/)
{
    return false;
}

bool
GSimulatorNullInterface::updateToGame(const GSimPlanet& /*pl*/)
{
    return false;
}

GSimulatorNullInterface::ShipRelation
GSimulatorNullInterface::getShipRelation(const GSimShip& /*sh*/)
{
    return ShipIsUnknown;
}

bool
GSimulatorNullInterface::selectGameShip(const GSimShip& /*sh*/)
{
    return false;
}
