/**
  *  \file resmgr/res-cc1.cc
  *
  *  (c) 2001-2006 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cstdlib>
#include "resmgr/res-cc1.h"
#include "util/nls.h"
#include "util/except.h"
#include "cpluslib/format.h"
#include "resmgr/cc1resid.h"
#include "resmgr/resid.h"

const TStructureParser ResourceStream::Entry::parser[] = {
    { T_UINT16, 1, offsetof(ResourceStream::Entry, id),  0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Entry, pos), 0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Entry, len), 0, 0 },
    { T_END, 0, 0, 0, 0 }
};

const TStructureParser ResourceStream::Header::parser[] = {
    { T_UINT16, 1, offsetof(ResourceStream::Header, magic),    0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Header, dirpos),   0, 0 },
    { T_UINT16, 1, offsetof(ResourceStream::Header, nentries), 0, 0 },
    { T_END, 0, 0, 0, 0 }
};

ResourceStream::ResourceStream(Ptr<Stream> aworker)
    : SubStreamBase(aworker)
{
    aworker->seek(0);
    if (!getStructure(*aworker, header)
        || header.magic != 0x5A52 /* 'RZ' */
        || !aworker->seek(header.dirpos))
    {
        throw FileFormatException(format(_("File `%s' is not a resource file"),
                                         aworker->getName()));
    }
    if (header.nentries) {
        uint32      rdsize = Entry::size * uint32(header.nentries);
        Array<char> tmp(rdsize);
        if (aworker->read(tmp.ptr(), rdsize) != rdsize)
            throw FileTooShortException(aworker->getName());
        
        rd.ensureSize(header.nentries);
        const char* p = tmp.ptr();
        for (unsigned i = 0; i < header.nentries; ++i)
            p = getStructure(p, rd[i]);
    }
    setDisallowedOperations(C_WRITE);
}

ResourceStream::~ResourceStream()
{ }

bool
ResourceStream::seekId(uint16 id)
{
    for (unsigned i = 0; i < header.nentries; ++i) {
        if (id == rd[i].id) {
            setRange(rd[i].pos, rd[i].len);
            seek(0);
            return true;
        }
    }
    return false;
}


ResProviderResFile::ResProviderResFile(Ptr<Stream> s)
    : res(new ResourceStream(s))
{ }

ResProviderResFile::ResProviderResFile(string_t name)
    : res(new ResourceStream(Stream::openFile(name, Stream::C_READ)))
{ }

ResProviderResFile::~ResProviderResFile()
{ }

void
ResProviderResFile::loadPixmap(ResManager& mgr, ResId id)
{
    int ele = -1;
    if (id.isElem(res::ship, 1))
        ele = cc1id_Ships + id.getId1();
    else if (id.isElem(res::base, 1))
        ele = (id.getId1() > 6 ? cc1id_Base_Hi : cc1id_Base_Lo);
    else if (id.isElem(res::vcr_ftr, 1))
        ele = cc1id_VCR_Fighter + id.getId1();
    else if (id.isElem("res", 1))
        ele = id.getId1();
    // FIXME: add more types

    if (ele >= 0 && res->seekId(ele))
        mgr.addPixmap(id, loadImage(res));
}
